@@ -10,9 +10,13 @@
 change log:
     2021/07/20  create file.
 """
+from dis import dis
+from functools import wraps
 from ..preprocess.qc import cal_qc
 from ..preprocess.filter import filter_cells, filter_genes, filter_coordinates
 from ..algorithm.normalization import normalize_total, quantile_norm, zscore_disksmooth
+from ..algorithm.scale import scale
+from ..algorithm.gaussian_smooth import gaussian_smooth
 import numpy as np
 from scipy.sparse import issparse
 from ..algorithm.dim_reduce import pca, u_map
@@ -25,13 +29,17 @@ from ..algorithm.leiden import leiden as le
 from ..algorithm._louvain import louvain as lo
 from typing_extensions import Literal
 from ..log_manager import logger
+from ..utils.time_consume import TimeConsume
 
+tc = TimeConsume()
 
 def logit(func):
+    @wraps(func)
     def wrapped(*args, **kwargs):
         logger.info('start to run {}...'.format(func.__name__))
+        tk = tc.start()
         res = func(*args, **kwargs)
-        logger.info('{} end.'.format(func.__name__))
+        logger.info('{} end, consume time {:.4f}s.'.format(func.__name__, tc.get_time_consumed(key=tk, restart=False)))
         return res
     return wrapped
 
@@ -76,6 +84,12 @@ class StPipeline(object):
         self.data = self.raw
 
     def raw_checkpoint(self):
+        """
+        Save the current results to self.raw.
+        :param key:
+        :param res_key:
+        :return:
+        """
         self.raw = self.data
 
     def reset_key_record(self, key, res_key):
@@ -179,6 +193,22 @@ class StPipeline(object):
         else:
             self.result[res_key] = normalize_total(self.data.exp_matrix, target_sum=target_sum)
 
+    @logit
+    def scale(self, zero_center=True, max_value=None, inplace=True, res_key='scale'):
+        """
+        scale to unit variance and zero mean for express matrix.
+
+        :param zero_center: ignore zero variables if `False`
+        :param max_value: truncate to this value after scaling. If `None`, do not truncate.
+        :param inplace: whether inplace the original data or get a new express matrix after scale.
+        :param res_key: the key for getting the result from the self.result.
+        :return:
+        """
+        if inplace:
+            self.data.exp_matrix = scale(self.data.exp_matrix, zero_center, max_value)
+        else:
+            self.result[res_key] = scale(self.data.exp_matrix, zero_center, max_value)
+
     @logit
     def quantile(self, inplace=True, res_key='quantile'):
         """
@@ -217,7 +247,7 @@ class StPipeline(object):
     def sctransform(self,
                     method="theta_ml",
                     n_cells=5000,
-                    n_genes=2000,
+                    n_genes=None,
                     filter_hvgs=False,
                     res_clip_range="seurat",
                     var_features_n=3000,
@@ -228,7 +258,7 @@ class StPipeline(object):
 
         :param method: offset, theta_ml, theta_lbfgs, alpha_lbfgs.
         :param n_cells: Number of cells to use for estimating parameters in Step1: default is 5000.
-        :param n_genes: Number of genes to use for estimating parameters in Step1; default is 2000.
+        :param n_genes: Number of genes to use for estimating parameters in Step1; default is None, means all genes.
         :param filter_hvgs: bool.
         :param res_clip_range: string or list
                     options: 1)"seurat": Clips residuals to -sqrt(ncells/30), sqrt(ncells/30)
@@ -318,12 +348,29 @@ class StPipeline(object):
         return data
 
     @logit
-    def pca(self, use_highly_genes, n_pcs, hvg_res_key='highly_variable_genes', res_key='pca'):
+    def pca(self, use_highly_genes, n_pcs, svd_solver='auto', hvg_res_key='highly_variable_genes', res_key='pca'):
         """
         Principal component analysis.
 
         :param use_highly_genes: Whether to use only the expression of hypervariable genes as input.
         :param n_pcs: the number of features for a return array after reducing.
+        :param svd_solver: {'auto', 'full', 'arpack', 'randomized'}, default to 'auto'
+                    If auto :
+                        The solver is selected by a default policy based on `X.shape` and
+                        `n_pcs`: if the input data is larger than 500x500 and the
+                        number of components to extract is lower than 80% of the smallest
+                        dimension of the data, then the more efficient 'randomized'
+                        method is enabled. Otherwise the exact full SVD is computed and
+                        optionally truncated afterwards.
+                    If full :
+                        run exact full SVD calling the standard LAPACK solver via
+                        `scipy.linalg.svd` and select the components by postprocessing
+                    If arpack :
+                        run SVD truncated to n_pcs calling ARPACK solver via
+                        `scipy.sparse.linalg.svds`. It requires strictly
+                        0 < n_pcs < min(x.shape)
+                    If randomized :
+                        run randomized SVD by the method of Halko et al.
         :param hvg_res_key: the key of highly varialbe genes to getting the result.
         :param res_key: the key for getting the result from the self.result.
         :return:
@@ -331,8 +378,7 @@ class StPipeline(object):
         if use_highly_genes and hvg_res_key not in self.result:
             raise Exception(f'{hvg_res_key} is not in the result, please check and run the highly_var_genes func.')
         data = self.subset_by_hvg(hvg_res_key, inplace=False) if use_highly_genes else self.data
-        x = data.exp_matrix.toarray() if issparse(data.exp_matrix) else data.exp_matrix
-        res = pca(x, n_pcs)
+        res = pca(data.exp_matrix, n_pcs, svd_solver=svd_solver)
         self.result[res_key] = pd.DataFrame(res['x_pca'])
         key = 'pca'
         self.reset_key_record(key, res_key)
@@ -400,14 +446,13 @@ class StPipeline(object):
         self.reset_key_record(key, res_key)
 
     @logit
-    def neighbors(self, pca_res_key, method='umap', metric='euclidean', n_pcs=None, n_neighbors=10, knn=True,
-                  res_key='neighbors'):
+    def neighbors(self, pca_res_key, method='umap', metric='euclidean', n_pcs=None, n_neighbors=10, knn=True, n_jobs=10, res_key='neighbors'):
         """
         run the neighbors.
 
         :param pca_res_key: the key of pca to getting the result.
         :param method: Use 'umap' or 'gauss'. for computing connectivities.
-        :param metric: A known metric’s name or a callable that returns a distance.
+        :param metric: A known metric's name or a callable that returns a distance.
                         include:
                             * euclidean
                             * manhattan
@@ -436,13 +481,15 @@ class StPipeline(object):
                     `n_neighbors`, that is, consider a knn graph. Otherwise, use a Gaussian
                     Kernel to assign low weights to neighbors more distant than the
                     `n_neighbors` nearest neighbor.
+        :param n_jobs: The number of parallel jobs to run for neighbors search, defaults to 10.
+                    if set to -1, means the all CPUs will be used, too high value may cause segment fault.
         :param res_key: the key for getting the result from the self.result.
         :return:
         """
         if pca_res_key not in self.result:
             raise Exception(f'{pca_res_key} is not in the result, please check and run the pca func.')
         neighbor, dists, connectivities = find_neighbors(x=self.result[pca_res_key].values, method=method, n_pcs=n_pcs,
-                                                         n_neighbors=n_neighbors, metric=metric, knn=knn)
+                                                         n_neighbors=n_neighbors, metric=metric, knn=knn, n_jobs=n_jobs)
         res = {'neighbor': neighbor, 'connectivities': connectivities, 'nn_dist': dists}
         self.result[res_key] = res
         key = 'neighbors'
@@ -517,7 +564,7 @@ class StPipeline(object):
         neighbor, connectivities, _ = self.get_neighbors_res(neighbors_res_key)
         clusters = le(neighbor=neighbor, adjacency=connectivities, directed=directed, resolution=resolution,
                       use_weights=use_weights, random_state=random_state, n_iterations=n_iterations)
-        df = pd.DataFrame({'bins': self.data.cell_names, 'group': clusters})
+        df = pd.DataFrame({'bins': self.data.cell_names, 'group': clusters}) 
         self.result[res_key] = df
         key = 'cluster'
         self.reset_key_record(key, res_key)
@@ -540,7 +587,7 @@ class StPipeline(object):
         :param resolution: A parameter value controlling the coarseness of the clustering.
                             Higher values lead to more clusters.
                             Set to `None` if overriding `partition_type`
-                            to one that doesn’t accept a `resolution_parameter`.
+                            to one that doesn't accept a `resolution_parameter`.
         :param random_state: Change the initialization of the optimization.
         :param flavor: Choose between to packages for computing the clustering.
                         Including: ``'vtraag'``, ``'igraph'``, ``'taynaud'``.
@@ -558,18 +605,21 @@ class StPipeline(object):
         self.reset_key_record(key, res_key)
 
     @logit
-    def phenograph(self, phenograph_k, pca_res_key, res_key='cluster'):
+    def phenograph(self, phenograph_k, pca_res_key, n_jobs=10, res_key='cluster'):
         """
         phenograph of cluster.
 
         :param phenograph_k: the k value of phenograph.
         :param pca_res_key: the key of pca to getting the result for running the phenograph.
+        :param n_jobs: The number of parallel jobs to run for neighbors search, defaults to 10.
+                    if set to -1, means the all CPUs will be used, too high value may cause segment fault.
         :param res_key: the key for getting the result from the self.result.
         :return:
         """
         if pca_res_key not in self.result:
             raise Exception(f'{pca_res_key} is not in the result, please check and run the pca func.')
-        communities, _, _ = phe.cluster(self.result[pca_res_key], k=phenograph_k, clustering_algo='leiden')
+        communities, _, _ = phe.cluster(self.result[pca_res_key], k=phenograph_k, clustering_algo='leiden', n_jobs=n_jobs)
+        communities = communities + 1
         clusters = communities.astype(str)
         df = pd.DataFrame({'bins': self.data.cell_names, 'group': clusters})
         self.result[res_key] = df
@@ -602,7 +652,7 @@ class StPipeline(object):
         :param use_highly_genes: Whether to use only the expression of hypervariable genes as input, default True.
         :param hvg_res_key: the key of highly varialbe genes to getting the result.
         :param res_key: the key for getting the result from the self.result.
-        :param output: file name for output. If None, do not generate the output file.
+        :param output: path of output_file(.csv). If None, do not generate the output file.
         :return:
         """
         from ..tools.find_markers import FindMarker
@@ -720,6 +770,103 @@ class StPipeline(object):
         # res = {"results":hs.results, "local_cor_z": hs.local_correlation_z, "modules": hs.modules,
         #        "module_scores": hs.module_scores}
         self.result[res_key] = hs
+    
+    @logit
+    def gaussian_smooth(self, n_neighbors=10, smooth_threshold=90, pca_res_key='pca', res_key='gaussian_smooth', n_jobs=-1, inplace=True):
+        """smooth the expression matrix
+
+        :param n_neighbors: number of the nearest points to serach, Too high value may cause overfitting, Too low value may cause poor smoothing effect.
+        :param smooth_threshold: indicates Gaussian variance with a value between 20 and 100, Too high value may cause overfitting, Too low value may cause poor smoothing effect。
+        :param pca_res_key: the key of pca to get from self.result, defaults to 'pca'.
+        :param res_key: the key for getting the result from the self.result, defaults to 'gaussian_smooth'.
+        :param n_jobs: The number of parallel jobs to run for neighbors search, defaults to -1, means the all CPUs will be used.
+        :param inplace: whether inplace the express matrix or get a new express matrix, defaults to True.
+        """
+        assert pca_res_key in self.result, f'{pca_res_key} is not in the result, please check and run the pca func.'
+        assert self.raw is not None, 'no raw exp_matrix to be saved, please check and run the raw_checkpoint.'
+        assert n_neighbors > 0, 'n_neighbors must be greater than 0'
+        assert smooth_threshold >= 20 and smooth_threshold <= 100, 'smooth_threshold must be between 20 and 100'
+
+        pca_exp_matrix = self.result[pca_res_key].values
+        raw_exp_matrix = self.raw.exp_matrix.toarray() if issparse(self.raw.exp_matrix) else self.raw.exp_matrix
+
+        if pca_exp_matrix.shape[0] != raw_exp_matrix.shape[0]:
+            raise Exception(f"The first dimension of pca_exp_matrix not equals to raw_exp_matrix's, may be because of running raw_checkpoint before filter cells and genes.")
+        
+        # logger.info(f"raw exp matrix size: {raw_exp_matrix.shape}")
+        result = gaussian_smooth(pca_exp_matrix, raw_exp_matrix, self.data.position, n_neighbors=n_neighbors, smooth_threshold=smooth_threshold, n_jobs=n_jobs)
+        # logger.info(f"smoothed exp matrix size: {result.shape}")
+        if inplace:
+            self.data.exp_matrix = result
+            cal_qc(self.data)
+        else:
+            self.result[res_key] = result
+                
+    def lr_score(
+        self, 
+        lr_pairs: Union[list, np.array],
+        distance: Union[int, float] = 5,
+        spot_comp: pd.DataFrame = None, 
+        verbose: bool = True, 
+        key_add: str = 'cci_score',
+        min_exp: Union[int, float] = 0, 
+        use_raw: bool = False,
+        min_spots: int = 20, 
+        n_pairs: int = 1000,
+        adj_method: str = "fdr_bh",
+        bin_scale: int = 1,
+        n_jobs=4,
+        res_key='lr_score'
+    ):
+        """calculate cci score for each LR pair and do permutation test
+
+        Parameters
+        ----------
+        lr_pairs : Union[list, np.array]
+            LR pairs
+        distance : Union[int, float], optional
+            the distance between spots which are considered as neighbors , by default 5
+        spot_comp : `pd.DataFrame`, optional
+            spot component of different cells, by default None
+        key_add : str, optional
+            key added in `result`, by default 'cci_score'
+        min_exp : Union[int, float], optional
+            the min expression of ligand or receptor gene when caculate reaction strength, by default 0
+        use_raw : bool, optional
+            whether to use counts in `adata.raw.X`, by default False
+        min_spots : int, optional
+            the min number of spots that score > 0, by default 20
+        n_pairs : int, optional
+            number of pairs to random sample, by default 1000
+        adj_method : str, optional
+            adjust method of p value, by default "fdr_bh"
+        n_wokers : int, optional
+            num of worker when calculate_score, by default 4
+
+        Raises
+        ------
+        ValueError
+            _description_
+        """
+        from ..tools.LR_interaction import LrInteraction
+        interaction = LrInteraction(self, 
+                                    verbose=verbose, 
+                                    bin_scale=bin_scale, 
+                                    distance=distance,
+                                    spot_comp=spot_comp,
+                                    n_jobs=n_jobs,
+                                    min_exp=min_exp,
+                                    min_spots=min_spots,
+                                    n_pairs=n_pairs,
+                                    )
+        
+        result = interaction.fit(lr_pairs=lr_pairs, 
+                                 adj_method=adj_method, 
+                                 use_raw=use_raw,
+                                 key_add=key_add)
+        
+        self.result[res_key] = result
+        
 
     # def scenic(self, tfs, motif, database_dir, res_key='scenic', use_raw=True, outdir=None,):
     #     """
