@@ -23,6 +23,8 @@ class GEF(object):
         self.cells = None
         self.cell_num = 0
         self.gene_num = 0
+        self.offset_x = 0
+        self.offset_y = 0
         self._init()
 
     def _init(self):
@@ -35,6 +37,8 @@ class GEF(object):
             h5gene = h5f['geneExp'][bin_tag]['gene']
             self.df_gene = pd.DataFrame(h5gene['gene', 'offset', 'count'])
             self.df_exp = pd.DataFrame(h5exp['x', 'y', 'count'])
+            self.offset_x = h5exp.attrs['minX']
+            self.offset_y = h5exp.attrs['minY']
 
     def build(self, gene_lst: list = None, region: list = None):
         if gene_lst is not None:
@@ -110,11 +114,13 @@ class GEF(object):
 
     def to_stereo_exp_data(self) -> StereoExpData:
         data = StereoExpData(file_path=self.file_path)
-        logger.info(f'the martrix has {self.cell_num} cells, and {self.gene_num} genes.')
+        logger.info(f'the matrix has {self.cell_num} cells, and {self.gene_num} genes.')
         data.position = self.df_exp.loc[:, ['x', 'y']].drop_duplicates().values
         exp_matrix = csr_matrix((self.df_exp['count'], (self.df_exp['cell_index'], self.df_exp['gene_index'])),
                                 shape=(self.cell_num, self.gene_num), dtype=np.int32)
         data.cells = Cell(cell_name=self.cells)
         data.genes = Gene(gene_name=self.genes)
         data.exp_matrix = exp_matrix if self.is_sparse else exp_matrix.toarray()
+        data.offset_x = self.offset_x
+        data.offset_y = self.offset_y
         return data
