@@ -6,11 +6,10 @@
 @file:__init__.py.py
 @time:2021/03/05
 """
-# from .clustering import plot_spatial_cluster
-# from .dim_reduce import plot_dim_reduce
-from .qc import spatial_distribution, genes_count, violin_distribution
-from .scatter import scatter
+from .violin import violin_distribution
+from .scatter import base_scatter
 from .marker_genes import marker_genes_text, marker_genes_heatmap
 from .plot_collection import PlotCollection
 from .interact_plot.spatial_cluster import interact_spatial_cluster
 from .interact_plot.interactive_scatter import InteractiveScatter
+from .interact_plot.poly_selection import PolySelection
